from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field


class Precipitation(BaseModel):
    type_ext: Optional[int] = Field(default=None)
    intensity: int
    amount: float
    type: int


class HPa(BaseModel):
    max: int
    min: int


class MmHgAtm(BaseModel):
    max: int
    min: int


class InHg(BaseModel):
    max: float
    min: float


class Pressure(BaseModel):
    h_pa: HPa
    mm_hg_atm: MmHgAtm
    in_hg: InHg


class Percent(BaseModel):
    max: int
    min: int
    avg: int


class Humidity(BaseModel):
    percent: Percent


class Description(BaseModel):
    full: str


class Cloudiness(BaseModel):
    type: int
    percent: int


class Date(BaseModel):
    utc: str = Field(..., alias="UTC")
    local: str
    time_zone_offset: int
    unix: int


class Radiation(BaseModel):
    max: int
    max_index: int


class Max(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Min(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Comfort(BaseModel):
    max: Max
    min: Min


class Max1(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Min1(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Water(BaseModel):
    max: Max1
    min: Min1


class Max2(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Min2(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Avg(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Air(BaseModel):
    max: Max2
    min: Min2
    avg: Avg


class Temperature(BaseModel):
    comfort: Comfort
    water: Water
    air: Air


class Max3(BaseModel):
    km_h: int
    m_s: int
    mi_h: int


class Min3(BaseModel):
    km_h: int
    m_s: int
    mi_h: int


class Avg1(BaseModel):
    km_h: Optional[int] = Field(default=None)
    m_s: Optional[int] = Field(default=None)
    mi_h: Optional[int] = Field(default=None)


class Speed(BaseModel):
    max: Max3
    min: Min3
    avg: Avg1


class Max4(BaseModel):
    degree: Optional[int] = Field(default=None)
    scale_8: Optional[int] = Field(default=None)


class Min4(BaseModel):
    degree: Optional[int] = Field(default=None)
    scale_8: Optional[int] = Field(default=None)


class Avg2(BaseModel):
    degree: Optional[int] = Field(default=None)
    scale_8: Optional[int] = Field(default=None)


class Direction(BaseModel):
    max: Max4
    min: Min4
    avg: Avg2


class Wind(BaseModel):
    speed: Speed
    direction: Direction


class ModelItem(BaseModel):
    precipitation: Precipitation
    pressure: Pressure
    humidity: Humidity
    gm: int
    description: Description
    cloudiness: Cloudiness
    date: Date
    phenomenon: Optional[int] = Field(default=None)
    radiation: Radiation
    city: int
    kind: Optional[str] = Field(default=None)
    storm: bool
    temperature: Temperature
    wind: Wind
    icon: str


class Model(BaseModel):
    __root__: List[ModelItem]



from __future__ import annotations

from typing import Tuple

from . import constants, types, validators


class BaseHttpClient:
    __slots__ = ("settings",)

    def __init__(self, settings: validators.Settings) -> None:
        self.settings = settings

    def _get_params_and_headers(
        self, params: types.Params
    ) -> Tuple[types.Params, types.Headers]:
        lang_dict = {"lang": self.settings.lang} if self.settings.lang else {}
        params_dict = params or {}
        token = self.settings.token or constants.DEFAULT_TOKEN
        return {**lang_dict, **params_dict}, {"X-Gismeteo-Token": token}



from __future__ import annotations

from typing import Tuple, Type

from .. import models, types, validators
from . import mixins_abc


class CurrentMixin(mixins_abc.PeriodMixin):
    __slots__ = ()

    def _get_params_by_coordinates(
        self, latitude: float, longitude: float
    ) -> Tuple[str, types.Params]:
        coords = validators.Coordinates(latitude=latitude, longitude=longitude)
        return self._endpoint, coords.dict()

    def _get_params_by_id(
        self,
        # pylint: disable-next=invalid-name,redefined-builtin
        id: int,
    ) -> Tuple[str, types.Params]:
        locality_id = validators.LocalityID.parse_obj(id)
        url = f"{self._endpoint}/{locality_id.__root__}"
        return url, None

    @property
    def _model(self) -> Type[models.current.Model]:
        return models.current.Model

    @property
    def _endpoint(self) -> str:
        return "weather/current"


class Step3Mixin(mixins_abc.StepNMixin):
    __slots__ = ()

    @property
    def _days_validator(self) -> Type[validators.Step3Days]:
        return validators.Step3Days

    @property
    def _model(self) -> Type[models.step3.Model]:
        return models.step3.Model

    @property
    def _endpoint(self) -> str:
        return "weather/forecast"


class Step6Mixin(mixins_abc.StepNMixin):
    __slots__ = ()

    @property
    def _days_validator(self) -> Type[validators.Step6or24Days]:
        return validators.Step6or24Days

    @property
    def _model(self) -> Type[models.step6.Model]:
        return models.step6.Model

    @property
    def _endpoint(self) -> str:
        return "weather/forecast/by_day_part"


class Step24Mixin(mixins_abc.StepNMixin):
    __slots__ = ()

    @property
    def _days_validator(self) -> Type[validators.Step6or24Days]:
        return validators.Step6or24Days

    @property
    def _model(self) -> Type[models.step24.Model]:
        return models.step24.Model

    @property
    def _endpoint(self) -> str:
        return "weather/forecast/aggregate"



from __future__ import annotations

from abc import ABCMeta, abstractmethod
from typing import Any, Type, Union

from .. import validators


class PeriodMixin(metaclass=ABCMeta):
    __slots__ = ()

    @property
    @abstractmethod
    def _model(self) -> Type[Any]:
        pass

    @property
    @abstractmethod
    def _endpoint(self) -> str:
        pass


class StepNMixin(PeriodMixin):
    __slots__ = ()

    @property
    @abstractmethod
    def _days_validator(
        self,
    ) -> Type[Union[validators.Step3Days, validators.Step6or24Days]]:
        pass



from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field


class Precipitation(BaseModel):
    type_ext: Optional[int] = Field(default=None)
    intensity: int
    correction: Optional[bool] = Field(default=None)
    amount: Optional[float] = Field(default=None)
    duration: int
    type: int


class Pressure(BaseModel):
    h_pa: int
    mm_hg_atm: int
    in_hg: float


class Humidity(BaseModel):
    percent: Optional[int] = Field(default=None)


class Direction(BaseModel):
    degree: Optional[int] = Field(default=None)
    scale_8: Optional[int] = Field(default=None)


class Speed(BaseModel):
    km_h: int
    m_s: int
    mi_h: int


class Wind(BaseModel):
    direction: Direction
    speed: Speed


class Cloudiness(BaseModel):
    type: int
    percent: int


class Date(BaseModel):
    utc: str = Field(..., alias="UTC")
    time_zone_offset: int
    local: str
    hr_to_forecast: Optional[int] = Field(default=None)
    unix: int


class Radiation(BaseModel):
    uvb_index: int
    uvb: int = Field(..., alias="UVB")


class Comfort(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Water(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Air(BaseModel):
    c: float = Field(..., alias="C")
    f: float = Field(..., alias="F")


class Temperature(BaseModel):
    comfort: Comfort
    water: Water
    air: Air


class Description(BaseModel):
    full: str


class ModelItem(BaseModel):
    precipitation: Precipitation
    pressure: Pressure
    humidity: Humidity
    icon: str
    gm: int
    wind: Wind
    cloudiness: Cloudiness
    date: Date
    phenomenon: Optional[int] = Field(default=None)
    radiation: Radiation
    city: int
    kind: str
    storm: bool
    temperature: Temperature
    description: Description


class Model(BaseModel):
    __root__: List[ModelItem]



from __future__ import annotations

from ipaddress import IPv4Address
from typing import Optional

from pydantic import BaseModel, Field

from . import types


class Settings(BaseModel):
    lang: Optional[types.Lang] = Field(...)
    token: Optional[str] = Field(...)

    class Config:
        anystr_strip_whitespace = True
        validate_assignment = True


class ImmutableModel(BaseModel):
    class Config:
        allow_mutation = False
        anystr_strip_whitespace = True


class Coordinates(ImmutableModel):
    latitude: float = Field(..., ge=-90, le=90)
    longitude: float = Field(..., ge=-180, le=180)


class SearchLimit(ImmutableModel):
    __root__: int = Field(..., ge=1, le=36)


class IPAddress(ImmutableModel):
    __root__: IPv4Address


class Query(ImmutableModel):
    __root__: str


class LocalityID(ImmutableModel):
    __root__: int = Field(..., ge=1)


class Step3Days(ImmutableModel):
    __root__: int = Field(..., ge=1, le=10)


class Step6or24Days(ImmutableModel):
    __root__: int = Field(..., ge=3, le=10)



from __future__ import annotations

from abc import ABCMeta, abstractmethod
from typing import Tuple, Type, Union

from .. import types, validators


class PeriodABC(metaclass=ABCMeta):
    __slots__ = ()

    @property
    @abstractmethod
    def _endpoint(self) -> str:
        pass


class StepNABC(PeriodABC):
    __slots__ = ()

    def _get_params_by_coordinates(
        self, latitude: float, longitude: float, *, days: Union[str, int]
    ) -> Tuple[str, types.Params]:
        coords = validators.Coordinates(latitude=latitude, longitude=longitude)
        days_model = self._days_validator.parse_obj(days)
        params = {"days": days_model.__root__, **coords.dict()}
        return self._endpoint, params

    def _get_params_by_id(
        self,
        # pylint: disable-next=invalid-name,redefined-builtin
        id: int,
        *,
        days: Union[str, int],
    ) -> Tuple[str, types.Params]:
        id_model = validators.LocalityID.parse_obj(id)
        url = f"{self._endpoint}/{id_model.__root__}"
        days_model = self._days_validator.parse_obj(days)
        params = {"days": days_model.__root__}
        return url, params

    @property
    @abstractmethod
    def _days_validator(
        self,
    ) -> Type[Union[validators.Step3Days, validators.Step6or24Days]]:
        pass



from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field


class District(BaseModel):
    name: str
    name_p: Optional[str] = Field(default=None, alias="nameP")


class Country(BaseModel):
    name: str
    code: str
    name_p: Optional[str] = Field(default=None, alias="nameP")


class SubDistrict(BaseModel):
    name: str
    name_p: Optional[str] = Field(default=None, alias="nameP")


class ModelItem(BaseModel):
    district: Optional[District] = Field(default=None)
    id: int
    sub_district: Optional[SubDistrict] = Field(default=None)
    url: str
    name_p: Optional[str] = Field(default=None, alias="nameP")
    name: str
    rate: int
    weight: int
    kind: str
    country: Country


class Model(BaseModel):
    __root__: List[ModelItem]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pygismeteo_base', 'pygismeteo_base.models', 'pygismeteo_base.periods']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.9,<2.0', 'typing-extensions>=3.7.4.3,<5']

setup_kwargs = {
    'name': 'pygismeteo-base',
    'version': '3.0.5',
    'description': 'Base for pygismeteo and aiopygismeteo',
    'long_description': '# pygismeteo-base\n\n[![CI](https://github.com/monosans/pygismeteo-base/actions/workflows/ci.yml/badge.svg?branch=main&event=push)](https://github.com/monosans/pygismeteo-base/actions/workflows/ci.yml)\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/monosans/pygismeteo-base/main.svg)](https://results.pre-commit.ci/latest/github/monosans/pygismeteo-base/main)\n\nБаза для [pygismeteo](https://github.com/monosans/pygismeteo) и [aiopygismeteo](https://github.com/monosans/aiopygismeteo).\n\n## License / Лицензия\n\n[MIT](https://github.com/monosans/pygismeteo-base/blob/main/LICENSE)\n',
    'author': 'monosans',
    'author_email': 'hsyqixco@protonmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/monosans/pygismeteo-base',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from . import types, validators


class Search:
    __slots__ = ()

    @staticmethod
    def _get_params_by_coordinates(
        latitude: float, longitude: float, *, limit: types.SearchLimit
    ) -> types.Params:
        coordinates = validators.Coordinates(
            latitude=latitude, longitude=longitude
        )
        params = coordinates.dict()
        lim = validators.SearchLimit.parse_obj(limit).__root__
        return {**params, "limit": lim}

    @staticmethod
    def _get_params_by_ip(ip: str) -> types.Params:
        ip = str(validators.IPAddress.parse_obj(ip).__root__)
        return {"ip": ip}

    @staticmethod
    def _get_params_by_query(query: str) -> types.Params:
        query = validators.Query.parse_obj(query).__root__
        return {"query": query}

    @property
    def _endpoint(self) -> str:
        return "search/cities"
